/*  
    BluePhone is a Bluetooth mobile phone manager
    Copyright (C) 2004 Ed Rayne <er241@cam.ac.uk>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <glib.h>

#include "sms.h"
#include "bluetooth.h"
#include "global.h"

/*
 * Large portions of code in this are taken from tsemgr and modified to suit my purposes
 * tsemgr (C) 2003 Sebastian Sareyko
 * http://tsemgr.sourceforge.net/
 */

static gint hexstr2int (gchar *hexstring, gint length);
static gchar *sms_data_7bit_decode (gchar *input, gint encoded_size);
static gchar *sms_filldata_7bit_decode (gchar *input, gint encoded_size);
static gchar *sms_data_7bit_encode (gchar *input, gint *encoded_size);
static gchar *convert_from_gsm (unsigned char *input);
static gchar *convert_to_gsm (unsigned char *input);

static gint
hexstr2int (gchar *hexstring, gint length)
{
	gint i, shift;
	gint digitvalue = 0;
	gint intvalue = 0;
	gchar hexdigit;
	
	for (i = 0; i < length; i++)
	{
		switch (hexdigit = hexstring[i])
		{
			case '1':
				digitvalue = 1;
				break;
			case '2':
				digitvalue = 2;
				break;
			case '3':
				digitvalue = 3;
				break;
			case '4':
				digitvalue = 4;
				break;
			case '5':
				digitvalue = 5;
				break;
			case '6':
				digitvalue = 6;
				break;
			case '7':
				digitvalue = 7;
				break;
			case '8':
				digitvalue = 8;
				break;
			case '9':
				digitvalue = 9;
				break;
			case 'A':
			case 'a':
				digitvalue = 10;
				break;
			case 'B':
			case 'b':
				digitvalue = 11;
				break;
			case 'C':
			case 'c':
				digitvalue = 12;
				break;
			case 'D':
			case 'd':
				digitvalue = 13;
				break;
			case 'E':
			case 'e':
				digitvalue = 14;
				break;
			case 'F':
			case 'f':
				digitvalue = 15;
				break;
			case '0':
			default:
				digitvalue = 0;
				break;
		}
		
		shift = 4 * (length - i - 1);  // multiplicator: <<4*  = *16^
		intvalue += digitvalue << shift;
	}
	return (intvalue);
}

static gchar * 
convert_from_gsm (unsigned char* input)
{
	gchar *retval;
	gint i;
	gint k = 0;
    
	retval = g_malloc ((strlen (input) + 1) * sizeof (gchar));
	for (i = 0;i < strlen (input); i++) 
	{
		switch (input[i])
		{
			default:
				retval[k] = input[i];
				break;
			case 0x80: // commercial at
				retval[k] =  0x0040;
				break;
			case 0x01: // pound sign
				retval[k] =  0x00a3;
				break;
			case 0x02: // dollar sign
				retval[k] =  0x0024;
				break;
			case 0x03: // yen sign
				retval[k] =  0x00a5;
				break;
			case 0x04: // small 'e' with grave
				retval[k] =  0x00e8;
				break;
			case 0x05: // small 'e' with acute
				retval[k] =  0x00e9;
				break;
			case 0x06: // small 'u' with grave
				retval[k] =  0x00f9;
				break;
			case 0x07: // small 'i' with grave
				retval[k] =  0x00ec;
				break;
			case 0x08: // small 'o' with grave
				retval[k] =  0x00f2;
				break;
			case 0x09: // small/capital 'c' with cedilla (we map to small)
				retval[k] =  0x00e7; // or 0x00c7 ?
				break;
			case 0x0b: // capital 'O' with stroke
				retval[k] =  0x00d8;
				break;
			case 0x0c: // small 'o' with stroke
				retval[k] =  0x00f8;
				break;
			case 0x0e: // capital 'A' with ring above
				retval[k] =  0x00c5;
				break;
			case 0x0f: // small 'a' with ring above
				retval[k] =  0x00e5;
				break;
			case 0x11: // underscore / low line
				retval[k] =  0x005f;
				break;
			case 0x1b:
				++i;
				switch (input[i])
				{
					default: // non-breaking space
						--i;
						retval[k] =  0x00a0;
						break;
					case 0x1a: // form feed
						retval[k] =  0x000c;
						break;
					case 0x14: // circumflex accent
						retval[k] =  0x005e;
						break;
					case 0x28: // left curly bracket
						retval[k] =  0x007b;
						break;
					case 0x29: // right curly bracket
						retval[k] = 0x007d;
						break;
					case 0x2f: // reverse solidus
						retval[k] = 0x005c;
						break;
					case 0x3c: // left square bracket
						retval[k] = 0x005b;
						break;
					case 0x3d: // tilde
						retval[k] = 0x007e;
						break;
					case 0x3e: // right square bracket
						retval[k] = 0x005d;
						break;
					case 0x40: // vertical line
						retval[k] = 0x007c;
						break;
				}
				break;
			case 0x1c: // capital 'AE'
				retval[k] =  0x00c6;
				break;
			case 0x1d: // small 'ae'
				retval[k] =  0x00e6;
				break;
			case 0x1e: // small sharp s (german)
				retval[k] =  0x00df;
				break;
			case 0x1f: // capital 'E' with acute
				retval[k] =  0x00c9;
				break;
			case 0x24: // currency sign
				retval[k] =  0x00a4;
				break;
			case 0x40: // inverted exclamation mark
				retval[k] =  0x00a1;
				break;
			case 0x5b: // capital 'A' with diaeresis
				retval[k] =  0x00c4;
				break;
			case 0x5c: // capital 'O' with diaeresis
				retval[k] =  0x00d6;
				break;
			case 0x5d: // capital 'N' with tilde
				retval[k] =  0x00d1;
				break;
			case 0x5e: // capital 'U' with diaeresis
				retval[k] =  0x00dc;
				break;
			case 0x5f: // section sign
				retval[k] =  0x00a7;
				break;
			case 0x60: // inverted question mark
				retval[k] =  0x00bf;
				break;
			case 0x7b: // small 'a' with diaeresis
				retval[k] =  0x00e4;
				break;
			case 0x7c: // small 'o' with diaeresis
				retval[k] =  0x00f6;
				break;
			case 0x7d: // small 'n' with tilde
				retval[k] =  0x00f1;
				break;
			case 0x7e: // small 'u' with diaeresis
				retval[k] =  0x00fc;
				break;
			case 0x7f: // small 'a' with grave
				retval[k] =  0x00e0;
				break;
		}
		++k;
	}
	retval[k]=0;
    	return g_realloc (retval, (k+1) * sizeof (gchar));
}


static gchar *
convert_to_gsm (unsigned char* input)
{
	unsigned char* retval;
	int i;
	int k = 0;
    
	retval = g_malloc ((strlen (input) * 2) + 1);
	for (i = 0; i < strlen (input); i++)
	{
		switch (input[i])
		{
			default:
				if (input[i] < 0x0080 && (
							  /* see Unicode table for descriptions */
				                          input[i] == 0x000A ||
			 	                          input[i] == 0x000D ||
				                          (input[i] >= 0x0020 && input[i] <= 0x0023) || 
				                          (input[i] >= 0x0025 && input[i] <= 0x003f) || 
				                          (input[i] >= 0x0041 && input[i] <= 0x005a) || 
				                          (input[i] >= 0x0061 && input[i] <= 0x007a)))
				{
					retval[k] = input[i];
				}
				else 
				{
					--k;
				}
				break;
			case 0x0040: // commercial at
				retval[k] =  0x80;
				break;
			case 0x00a3: // pound sign
				retval[k] =  0x01;
				break;
			case 0x0024: // dollar sign
				retval[k] =  0x02;
				break;
			case 0x00a5: // yen sign
				retval[k] =  0x03;
				break;
			case 0x00e8: // small 'e' with grave
				retval[k] =  0x04;
				break;
			case 0x00e9: // small 'e' with acute
				retval[k] =  0x05;
				break;
			case 0x00f9: // small 'u' with grave
				retval[k] =  0x06;
				break;
			case 0x00ec: // small 'i' with grave
				retval[k] =  0x07;
				break;
			case 0x00f2: // small 'o' with grave
				retval[k] =  0x08;
				break;
			case 0x00c7: // capital 'C' with cedilla
				case 0x00e7: // small 'c' with cedilla
				retval[k] =  0x09;
				break;
			case 0x00d8: // capital 'O' with stroke
				retval[k] =  0x0b;
				break;
			case 0x00f8: // small 'o' with stroke
				retval[k] =  0x0c;
				break;
			case 0x00c5: // capital 'A' with ring above
				retval[k] =  0x0e;
				break;
			case 0x00e5: // small 'a' with ring above
				retval[k] =  0x0f;
				break;
			case 0x0394: // capital delta
				retval[k] =  0x10;
				break;
			case 0x005f: // underscore / low line
				retval[k] =  0x11;
				break;
			case 0x03a6: // capital phi
				retval[k] =  0x12;
				break;
			case 0x0393: // capital gamma
				retval[k] =  0x13;
				break;
			case 0x039b: // capital lambda
				retval[k] =  0x14;
				break;
			case 0x03a9: // capital omega
				retval[k] =  0x15;
				break;
			case 0x03a0: // capital pi
				retval[k] =  0x16;
				break;
			case 0x03a8: // capital psi
				retval[k] =  0x17;
				break;
			case 0x03a3: // capital sigma
				retval[k] =  0x18;
				break;
			case 0x0398: // capital theta
				retval[k] =  0x19;
				break;
			case 0x039e: // capital xi
				retval[k] =  0x1a;
				break;
			case 0x000c: // form feed
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x0a;
				break;
			case 0x005e: // circumflex accent
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x14;
				break;
			case 0x007b: // left curly bracket
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x28;
				break;
			case 0x007d: // right curly bracket
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x29;
				break;
			case 0x005c: // reverse solidus (back slash)
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x2f;
				break;
			case 0x005b: // left square bracket
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x3c;
				break;
			case 0x007e: // tilde
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x3d;
				break;
			case 0x005d: // right square bracket
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x3e;
				break;
			case 0x007c: // vertical line
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x40;
				break;
			case 0x20ac: // Euro sign
				retval[k] =  0x1b;
				++k;
				retval[k] =  0x65;
				break;
			case 0x00c6: // capital 'AE'
				retval[k] =  0x1c;
				break;
			case 0x00e6: // small 'ae'
				retval[k] =  0x1d;
				break;
			case 0x00df: // small sharp s (german)
				retval[k] =  0x1e;
				break;
			case 0x00c9: // capital 'E' with acute
				retval[k] =  0x1f;
				break;
			case 0x00a4: // currency sign
			case 0x00b0: // degree sign (not offical, preference of author)
				retval[k] =  0x24;
				break;
			case 0x00a1: // inverted exclamation mark
				retval[k] =  0x40;
				break;
			case 0x0391: // capital alpha
				retval[k] =  0x41;
				break;
			case 0x0392: // capital beta
				retval[k] =  0x42;
				break;
			case 0x0395: // capital epsilon
				retval[k] =  0x45;
				break;
			case 0x0397: // capital eta
				retval[k] =  0x48;
				break;
			case 0x0399: // capital iota
				retval[k] =  0x49;
				break;
			case 0x039a: // capital kappa
				retval[k] =  0x4b;
				break;
			case 0x039c: // capital mu
				retval[k] =  0x4d;
				break;
			case 0x039d: // capital nu
				retval[k] =  0x4e;
				break;
			case 0x039f: // capital omicron
				retval[k] =  0x4f;
				break;
			case 0x03a1: // capital rho
				retval[k] =  0x50;
				break;
			case 0x03a4: // capital tau
				retval[k] =  0x54;
				break;
			case 0x03a5: // capital upsilon
				retval[k] =  0x55;
				break;
			case 0x03a7: // capital chi
				retval[k] =  0x58;
				break;
			case 0x0396: // capital zeta
				retval[k] =  0x5a;
				break;	
			case 0x00c4: // capital 'A' with diaeresis
				retval[k] =  0x5b;
				break;
			case 0x00d6: // capital 'O' with diaeresis
				retval[k] =  0x5c;
				break;
			case 0x00d1: // capital 'N' with tilde
				retval[k] =  0x5d;
				break;
			case 0x00dc: // capital 'U' with diaeresis
				retval[k] =  0x5e;
				break;
			case 0x00a7: // section sign
				retval[k] =  0x5f;
				break;
			case 0x00bf: // inverted question mark
				retval[k] =  0x60;
				break;
			case 0x00e4: // small 'a' with diaeresis
				retval[k] =  0x7b;
				break;
			case 0x00f6: // small 'o' with diaeresis
				retval[k] =  0x7c;
				break;
			case 0x00f1: // small 'n' with tilde
				retval[k] =  0x7d;
				break;
			case 0x00fc: // small 'u' with diaeresis
				retval[k] =  0x7e;
				break;
			case 0x00e0: // small 'a' with grave
				retval[k] =  0x7f;
				break;
		}
		++k;
	}
	retval[k] = 0;
	return g_realloc (retval , k + 1);
}

static gchar * 
sms_data_7bit_decode (gchar *input, gint encoded_size)
{
	unsigned char userdata[281];
	unsigned char decoded[161];
	gint i, first, end, offset, counter;

	if (!encoded_size || input == NULL)
		return NULL;

	counter = 0;
	for (i = 0; i < strlen (input); i += 2)
		userdata[counter++] = hexstr2int (input + i, 2);
	offset = -1;
	counter = 0;
	for (i = 0; i < encoded_size; i++)
	{
		if (i % 8)
		{
			first = userdata[i - 1 - offset];
			first >>= 8 - (i % 8);
			end = userdata[i - offset];
			end <<= (i % 8);
			*(decoded + counter) = (first | end) & 127;
		}
		else
		{
			++offset;
			*(decoded + counter) = userdata[i - offset] & 127;
		}
		if (*(decoded + counter) == 0) /* special char @*/
			*(decoded + counter) = 128;
    		++counter;
	}
	decoded[counter] = '\0';
	return convert_from_gsm (decoded);
}

static gchar * 
sms_filldata_7bit_decode (gchar *input, gint encoded_size)
{
	unsigned char userdata[281];
	unsigned char decoded[161];
	gint i, first, tmp, last, counter, ls;

	if (input == NULL || encoded_size == 0)
		return NULL;

	counter = 0;
	for (i = 0; i < strlen (input); i += 2)
		userdata[counter++] = hexstr2int (input + i, 2);
	first = 0;
	last = 0;
	ls = 6;
	counter = 1;
	tmp = 1;
	for (i = encoded_size; i > 0;)
	{
		--i;
		first = userdata[i];
		first >>= counter;
		*(decoded + i) = (first | last) & 127;
		last = userdata[i] & tmp;
		last <<= ls;
		--ls;
		tmp = (tmp * 2) + 1;
		++counter;
	}
	decoded[counter - 1] = '\0';
	return convert_from_gsm (decoded);
}

static gchar *
sms_data_7bit_encode (gchar *input, gint *encoded_size)
{
	unsigned char *temp;
	gint i, first, end, outsize;
	gchar *retval;

	if (input == NULL || encoded_size == NULL)
		return NULL;
	temp = convert_to_gsm (input);

	*encoded_size = strlen (temp);
	outsize = 0;
	retval = g_strdup ("");
	// 7bit-GSM octet encoding
	for (i = 0; i < strlen (temp); i++)
	{
		end = temp[i] & 127; // the character are 7bit
		end >>= (i % 8);
		first = temp[i+1] & 127;
		first <<= 7 - (i % 8);
		first &= 255; // we only need the last eight bits (AND mask)
		retval = g_strdup_printf ("%s%02X", retval, (first | end));
		++outsize;
		if ((i % 8) == 6) // every 8th char is already coded in (it really must be 6)
		{
			i++;
		}
	}
	return retval;
}

/*
 * this function can be tidied up a lot
 * need to get round to doing it before cvs submission
 */
void
send_sms_message (gchar *message, gchar *number)
{
	gchar *encoded, *buffer, *temp, *tempmsg, *messageflags, *command, *iobuffer;
	gint total, len, i, j;
	gint k = 0;
	gint ret = 1;
	gint stop = 0;
	GIOChannel *channel;

	/* number of messages to send */
	total = strlen (message) / 160;
	if (strlen (message) % 160)
		total++;

	/* loop through each message */
	for (i = 0; i <= total; i++)
	{
		/* set the message flags */
		if (strlen (message) > 160)
			messageflags = g_strdup_printf ("41");
		else
			messageflags = g_strdup_printf ("01");

		/* build up the SMS message */
		buffer = g_strdup_printf ("00"); /* SMSC length */
		buffer = g_strdup_printf ("%s%s", buffer, messageflags); /* message flags */
		buffer = g_strdup_printf ("%s00", buffer); /* message reference number */
		if (number[0] == '+')
		{
			/* if +44 format */
			buffer = g_strdup_printf ("%s%02X", buffer, strlen (number) - 1); /* number length */
			buffer = g_strdup_printf ("%s91", buffer); /* number type */
			number = g_strdup_printf ("%s", number + 1); /* trim initial + */
		}
		else
		{
			buffer = g_strdup_printf ("%s%02X", buffer, strlen (number)); /* number length */
			buffer = g_strdup_printf ("%s81", buffer); /* number type */
		}
		if (strlen (number) % 2 != 0)
			number = g_strdup_printf ("%sF", number); /* make number even number of bytes */
		temp = g_strdup_printf ("%c%c", number[1], number[0]);
		for (j = 2; j < strlen (number); j += 2)
			temp = g_strdup_printf ("%s%c%c", temp, number[j+1], number[j]);
		buffer = g_strdup_printf ("%s%s", buffer, temp); /* address number */
		buffer = g_strdup_printf ("%s00", buffer); /* protocol identifier */
		buffer = g_strdup_printf ("%s00", buffer); /* data coding scheme */
		/* pick out 160 characters to submit */
		if (strlen (message + k) >= 160)
			tempmsg = g_strndup (message + k, 160);
		else
			tempmsg = g_strndup (message + k, strlen (message + k));
		encoded = sms_data_7bit_encode (tempmsg, &len);
		if (strlen (message) > 160)
		{
			buffer = g_strdup_printf ("%s%02X", buffer, len + 6); /* ud length + udh length */
			buffer = g_strdup_printf ("%s%02X", buffer, 6); /* udh length */
			buffer = g_strdup_printf ("%s%02X", buffer, 8); /* iei */
			buffer = g_strdup_printf ("%s%02X", buffer, 4); /* ie length */
			buffer = g_strdup_printf ("%s%02X", buffer, 0); /* ie0 (16bit reference part I) */
			buffer = g_strdup_printf ("%s%02X", buffer, 23); /* ie1 (16bit referene part II) */
			buffer = g_strdup_printf ("%s%02X", buffer, total); /* ie2 (total parts) */
			buffer = g_strdup_printf ("%s%02X", buffer, i); /* ie3 (this part) */
		}
		else
			buffer = g_strdup_printf ("%s%02X", buffer, len); /* ud length */
		buffer = g_strdup_printf ("%s%s", buffer, encoded); /* pdu */

		/* let's send the message */
		channel = g_io_channel_unix_new (global.socket);
		g_io_channel_set_encoding (channel, NULL, NULL);
		
		command = g_strdup_printf ("AT+CMGS=%d\r", (strlen (buffer) / 2) - 1);
		rfcomm_command (command);
		for (;;)
		{
			buffer = rfcomm_get ();
			if (strstr (buffer, ">"))
				break;
			if (strstr (buffer, "ERROR"))
				break;
		}

		command = g_strdup_printf ("%s\x1a", buffer);
		rfcomm_command (command);
		
		/* free up all the pointers for next round */
		g_free (buffer);
		g_free (temp);
		g_free (encoded);
		g_free (tempmsg);
		g_free (command);
		g_free (iobuffer);
		
		k += 160;
	}
}

sms_t *
decode_sms_message (gchar *pdu)
{
	gint i, j;
	gint smsclength;
	gint udh;
	gint senderlength;
	gint udhlength;
	gchar *tmp;
	gboolean senderodd = FALSE;
	sms_t *sms;
	
	sms = g_malloc (sizeof (sms_t));
	
	/* get the SMS centre number length */
	smsclength = hexstr2int (pdu, 2);

	/* get the udh octet */
	i = 2 + (smsclength * 2);
	udh = hexstr2int (pdu + i, 2);
	
	/* get the sender number length */
	i += 2;
	senderlength = hexstr2int (pdu + i, 2);

	/* get the sender number type */
	i += 2;
	sms->sendertype = hexstr2int (pdu + i, 2);

	/* is the address odd or even? */
	if (senderlength & 1)
	{
		senderlength++;
		senderodd = TRUE;
	}

	/* decode the sender's number */
	i += 2;
	if ((sms->sendertype >> 4) == 13) /* alphanumeric */
	{
		senderlength /= 2;
		sms->sender = sms_data_7bit_decode (pdu + i, senderlength);
		j = senderlength * 2;
	}
	else if ((sms->sendertype >> 4) == 9) /* international */
	{
		sms->sender = g_malloc0 (sizeof (sms->sender) * (senderlength + 1));
		for (j = 0; j < senderlength; j += 2)
		{
			sms->sender[j] = pdu[i + j + 1];
			sms->sender[j + 1] = pdu[i + j];
		}
		if (senderodd == TRUE)
			sms->sender[senderlength - 1] = '\0';
		else
			sms->sender[senderlength] = '\0';
		sms->sender = g_strdup_printf ("+%s", sms->sender);
	}
	else if ((sms->sendertype >> 4) == 10) /* national */
	{
		sms->sender = g_malloc0 (sizeof (sms->sender) * (senderlength + 1));
		for (j = 0; j < senderlength; j += 2)
		{
			sms->sender[j] = pdu[i + j + 1];
			sms->sender[j + 1] = pdu[i + j];
		}
		if (senderodd == TRUE)
			sms->sender[senderlength - 1] = '\0';
		else
			sms->sender[senderlength] = '\0';
	}
	else
		sms->sender = g_strdup ("Unknown number format");

	/* get timestamp */
	i += (senderlength + 4);
	sms->timestamp = g_strdup_printf ("%c%c.%c%c.%c%c, %c%c:%c%c.%c%c",
	 		                 pdu[i+5], pdu[i+4],	/* date */
			    	         pdu[i+3], pdu[i+2],	/* month */
				         pdu[i+1], pdu[0],	/* year */
				         pdu[i+7], pdu[i+6],	/* hours */
				         pdu[i+9], pdu[i+8],	/* minutes */
				         pdu[i+11], pdu[i+10]);	/* seconds */

	/* message length */
	i += 14;
	sms->messagelength = hexstr2int (pdu + i, 2);

	/* check for udh data */
	i += 2;
	if (udh & 0x40)
	{
		/* don't want to bother decoding udh data */
		udhlength = hexstr2int (pdu + i, 2);
		i += (udhlength * 2) + 2;
	}
	if ((udh & 0x40) && ((1120 - ((udhlength + 1) * 8)) % 7))
	{
		/* we'd probably better shorten the message to account for udhdata */
		for (j = 0; ((1120 - ((udhlength + 1) * 8)) % 7) != 0; j++)
		{
			(sms->messagelength)--;
			i += 2;
		}
	}
	
	/* finally get and decode the message */
	if (sms->messagelength + j > 0)
	{
		if (sms->messagelength > 0)
			sms->message = sms_data_7bit_decode (pdu + i, sms->messagelength);
		else
			sms->message = g_strdup ("");
		
		if ((udh & 0x40) && ((1120 - ((udhlength + 1) * 8)) % 7))
		{
			tmp = sms_filldata_7bit_decode (pdu + i - (j * 2), j);
			sms->message = g_strdup_printf ("%s%s", tmp, sms->message);
			g_free (tmp);
			sms->messagelength += j;
		}
	}
	else
	{
		sms->messagelength = 0;
		sms->message = g_strdup ("");
	}
	return sms;
}
